@@ -18,8 +18,8 @@ from ixmp4.conf.auth import ManagerAuth
 from requests.auth import AuthBase
 
 from pyam.core import IamDataFrame
+from pyam.exceptions import deprecation_warning
 from pyam.ixmp4 import read_ixmp4
-from pyam.logging import deprecation_warning
 from pyam.str import is_str
 from pyam.utils import (
     IAMC_IDX,
@@ -54,10 +54,6 @@ def platforms() -> None:
     tabulate_manager_platforms(ixmp4.conf.settings.manager.list_platforms())
 
 
-def set_config(*args, **kwargs):
-    raise DeprecationWarning(f"This method is deprecated. {IXMP4_LOGIN}.")
-
-
 def _read_config(file):
     """Read username and password for IIASA API connection from file"""
     with open(file) as stream:
@@ -581,7 +577,7 @@ class Connection:
 
 
 def _new_default_api(kwargs):
-    # TODO: argument `default` is deprecated, remove this warning for release >= 3.0
+    # TODO: argument `default` is deprecated, remove this warning for release >= 3.1
     raise DeprecationWarning(
         "The argument `default` is deprecated, use `default_only` instead."
     )
