@@ -1,15 +1,24 @@
 import pandas.testing as pdt
 
+from pyam import IamDataFrame
+
 from . import compare
 
 
-def assert_iamframe_equal(left, right, **kwargs):
+def assert_iamframe_equal(
+    left: IamDataFrame,
+    right: IamDataFrame,
+    check_meta: bool = True,
+    **kwargs,
+) -> None:
     """Check that left and right IamDataFrame instances are equal.
 
     Parameters
     ----------
     left, right : :class:`IamDataFrame`
         Two IamDataFrame instances to be compared.
+    check_meta: bool
+        Whether to check that the `meta` indicators are identical.
     **kwargs
         Passed to :meth:`IamDataFrame.compare`, comparing the `data` objects.
 
@@ -26,13 +35,14 @@ def assert_iamframe_equal(left, right, **kwargs):
         msg = "IamDataFrame.data are different: \n {}"
         raise AssertionError(msg.format(diff.head()))
 
-    pdt.assert_frame_equal(
-        left.meta.dropna(axis="columns", how="all"),
-        right.meta.dropna(axis="columns", how="all"),
-        check_column_type=False,
-        check_dtype=False,
-        check_like=True,
-    )
+    if check_meta:
+        pdt.assert_frame_equal(
+            left.meta.dropna(axis="columns", how="all"),
+            right.meta.dropna(axis="columns", how="all"),
+            check_column_type=False,
+            check_dtype=False,
+            check_like=True,
+        )
 
     pdt.assert_series_equal(
         left.exclude,
