@@ -1,16 +1,21 @@
+import importlib.metadata
 import itertools
 import logging
 import re
 import string
+import warnings
+from contextlib import contextmanager
+from datetime import datetime, timedelta
 from pathlib import Path
 
 import dateutil
 import numpy as np
+import packaging.version
 import pandas as pd
 from pandas.api.types import is_list_like
 
+from pyam.exceptions import raise_data_error
 from pyam.index import get_index_levels, replace_index_labels
-from pyam.logging import raise_data_error
 from pyam.str import concat_with_pipe, escape_regexp, find_depth, is_str
 
 logger = logging.getLogger(__name__)
@@ -27,7 +32,7 @@ LONG_IDX = IAMC_IDX + ["year"]
 REQUIRED_COLS = ["region", "variable", "unit"]
 
 # illegal terms for data/meta column names to prevent attribute conflicts
-ILLEGAL_COLS = ["data", "meta", "level", "exclude", ""]
+ILLEGAL_COLS = ["data", "meta", "level", "exclude", "measurand", ""]
 
 # dictionary to translate column count to Excel column names
 NUMERIC_TO_STR = dict(
@@ -43,13 +48,16 @@ NUMERIC_TO_STR = dict(
     )
 )
 
-KNOWN_FUNCS = {
-    "min": np.min,
-    "max": np.max,
-    "avg": np.mean,
-    "mean": np.mean,
-    "sum": "sum",
-}
+
+@contextmanager
+def adjust_log_level(logger="pyam", level="ERROR"):
+    """Context manager to change loglevel"""
+    if isinstance(logger, str):
+        logger = logging.getLogger(logger)
+    old_level = logger.getEffectiveLevel()
+    logger.setLevel(level)
+    yield
+    logger.setLevel(old_level)
 
 
 def to_list(x):
@@ -82,7 +90,7 @@ def write_sheet(writer, name, df, index=False):
     """
     if index:
         df = df.reset_index()
-    df.to_excel(writer, name, index=False)
+    df.to_excel(writer, sheet_name=name, index=False)
     for i, col in enumerate(df.columns):
         if df.dtypes[col].name.startswith(("float", "int")):
             width = len(str(col)) + 2
@@ -94,13 +102,36 @@ def write_sheet(writer, name, df, index=False):
         writer.sheets[name].set_column(i, i, width)  # assumes xlsxwriter as engine
 
 
+def get_excel_file_with_kwargs(path, **kwargs):
+    """Return a `pandas.ExcelFile` and a dict of unused kwargs.
+
+    When reading an Excel file, this function finds keyword arguments that
+    should be passed to `pandas.ExcelFile`, and returns a `pandas.ExcelFile`
+    instance along with the remaining keyword arguments (which presumably
+    will be used for other purposes by the calling function).
+    """
+    EXCEL_FILE_KWS = ("engine", "storage_options", "engine_kwargs")
+    kwargs = kwargs.copy()
+    excel_file_kwargs = {k: kwargs.pop(k) for k in EXCEL_FILE_KWS if k in kwargs}
+    # TODO remove when bumping minimum pandas dependency to >= 2.2
+    if "engine_kwargs" in excel_file_kwargs and packaging.version.parse(
+        importlib.metadata.version("pandas")
+    ) < packaging.version.parse("2.2.0"):
+        warnings.warn(
+            "pandas < 2.2.0 has inconsistent support for `engine_kwargs`. "
+            "Using it is likely to result in an exception."
+        )
+    return pd.ExcelFile(path, **excel_file_kwargs), kwargs
+
+
 def read_pandas(path, sheet_name=["data*", "Data*"], *args, **kwargs):
     """Read a file and return a pandas.DataFrame"""
 
     if isinstance(path, Path) and path.suffix == ".csv":
         return pd.read_csv(path, *args, **kwargs)
 
-    with pd.ExcelFile(path) as xl:
+    xlfile, kwargs = get_excel_file_with_kwargs(path, **kwargs)
+    with xlfile as xl:
         # reading multiple sheets
         sheet_names = pd.Series(xl.sheet_names)
         if len(sheet_names) > 1:
@@ -315,7 +346,7 @@ def _format_data_to_series(df, index):
         df = (
             df.set_index(index + REQUIRED_COLS + extra_cols)
             .rename_axis(columns=time_col)
-            .stack(dropna=True)
+            .stack()
             .rename("value")
             .reorder_levels(idx_order)
         )
@@ -326,6 +357,30 @@ def _format_data_to_series(df, index):
     return df, time_col, extra_cols
 
 
+def _check_data(df, time_col):
+    """Validate data series for infinite values, duplicates, and emptiness"""
+
+    # check for infinite values
+    inf_rows = np.isinf(df)
+    if inf_rows.any():
+        raise_data_error(
+            "Infinite values in `data`", df[inf_rows].index.to_frame(index=False)
+        )
+
+    # check for duplicate rows
+    rows = df.index.duplicated()
+    if any(rows):
+        raise_data_error(
+            "Duplicate rows in `data`", df[rows].index.to_frame(index=False)
+        )
+
+    # check if data is empty
+    if df.empty:
+        logger.warning("Formatted data is empty.")
+
+    return df
+
+
 def format_data(df, index, **kwargs):  # noqa: C901
     """Convert a pandas.Dataframe or pandas.Series to the required format"""
 
@@ -398,16 +453,10 @@ def format_data(df, index, **kwargs):  # noqa: C901
     _time = [to_time(i) for i in get_index_levels(df.index, time_col)]
     df.index = replace_index_labels(df.index, time_col, _time)
 
-    rows = df.index.duplicated()
-    if any(rows):
-        raise_data_error(
-            "Duplicate rows in `data`", df[rows].index.to_frame(index=False)
-        )
-    del rows
-    if df.empty:
-        logger.warning("Formatted data is empty.")
+    # perform data validation checks
+    df = _check_data(df, time_col)
 
-    return df.sort_index(), index, time_col, extra_cols
+    return df, index, time_col, extra_cols
 
 
 def _validate_complete_index(df):
@@ -422,11 +471,6 @@ def _validate_complete_index(df):
         )
 
 
-def sort_data(data, cols):
-    """Sort data rows and order columns by cols"""
-    return data.sort_values(cols)[cols + ["value"]].reset_index(drop=True)
-
-
 def merge_meta(left, right, ignore_conflict=False):
     """Merge two `meta` tables; raise if values are in conflict (optional)
 
@@ -582,6 +626,19 @@ def print_list(x, n):
     return lst + count
 
 
+# utility method to compare years (as integer) and datetime for index-sorting
+def compare_year_time(x):
+    return pd.Index(
+        [
+            # set year lower than first timestep of that year (2010 < 2010-01-01 00:00)
+            datetime(time, 1, 1, 0, 0, 0) - timedelta(0, 0.01)
+            if isinstance(time, int)
+            else time
+            for time in x
+        ]
+    )
+
+
 def to_time(x):
     """Cast a value to either year (int) or datetime"""
 
